/*
 Copyright 2015 Forward Computing and Control Pty. Ltd.
 ACN 003 669 994, NSW, Australia     All rights Reserved

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

 http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.

 Further more 
 a) Neither the name of the author nor Forward Computing and Control may be 
 used to endorse or promote products derived from this software
 without specific prior written permission.

 b) All advertising materials mentioning features or use of this
 software must display the following acknowledgment: 
 "This product includes software developed by Matthew Ford
 and Forward Computing and Control Pty. Ltd. 
 (http://www.forward.com.au)."


 THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
 */

/*
 * MainFrame.java
 *
 */
package au.com.forward.pfodQRpsk;

import gnu.crypto.Registry;
import gnu.crypto.mac.HMac;
import gnu.crypto.mac.HMacFactory;
import gnu.crypto.mac.IMac;
import gnu.crypto.util.Util;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.StringSelection;
import java.awt.datatransfer.Transferable;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionListener;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.math.BigInteger;
import java.util.HashMap;
import javax.swing.AbstractAction;
import javax.swing.JDialog;
import javax.swing.JMenuBar;
import javax.swing.JPopupMenu;
import javax.swing.SwingUtilities;
import net.glxn.qrgen.QRCode;
import net.glxn.qrgen.image.ImageType;

public class MainFrame extends javax.swing.JFrame implements java.awt.datatransfer.ClipboardOwner {

    private int minBytes = ApplicationSettings.MIN_BYTES;

    private JDialog aboutBox;
    private javax.swing.JMenuBar menuBar;
    private Clipboard clipboard = getToolkit().getSystemClipboard();
    private JPopupMenu popupPasteMenu = new javax.swing.JPopupMenu();
    private JPopupMenu popupCopyMenu = new javax.swing.JPopupMenu();
    private String StandardMsg = "Move the mouse in the text area below to initialize the key gennerator.";
    private String OKMsg = "Press Generate Key to create secret key.";
    private String AnotherKeyMsg = "Key Generated.";
    private javax.swing.JMenuItem exitMenuItem = new javax.swing.JMenuItem();
    private javax.swing.JMenu helpMenu = new javax.swing.JMenu();
    private javax.swing.JMenuItem aboutMenuItem = new javax.swing.JMenuItem();
    private javax.swing.JMenu fileMenu = new javax.swing.JMenu();
    private long time1;
    private String hashCodeResult;
    private byte[] mouseBytes = new byte[minBytes];
    private int mouseBytesIndex = 0;
    private boolean keyGenerated = false;

    public void exit() {
        System.exit(0);
    }

    private static final char[] HEXDIGITS = "0123456789ABCDEF".toCharArray();

    /**
     * for debug
     *
     * @param bArray
     * @param len
     * @return
     */
    public static String byteArrayToHexString(byte[] bArray, int len) {
        if (bArray.length < len) {
            throw new IllegalArgumentException("len:" + len + " longer the array length:" + bArray.length);
        }
        StringBuilder sb = new StringBuilder(len * 2);
        for (int i = 0; i < len; i++) {
            sb.append(HEXDIGITS[(bArray[i] >>> 4) & 0x0f]);
            sb.append(HEXDIGITS[(bArray[i]) & 0x0f]);
        }
        return sb.toString();
    }

    public void setMenuBar(JMenuBar menuBar) {
        JMenuBar oldValue = getCurrentMenuBar();
        this.menuBar = menuBar;
        getRootPane().setJMenuBar(menuBar);
        firePropertyChange("menuBar", oldValue, menuBar);
    }

    public JMenuBar getCurrentMenuBar() {
        return menuBar;
    }

    public void showAboutBox() {
        if (aboutBox == null) {
            aboutBox = new AboutDialog(this, true);
            aboutBox.pack();
        }
        Dimension dims = Toolkit.getDefaultToolkit().getScreenSize();
        int wscreen = dims.width;
        int hscreen = dims.height;

        int ws = aboutBox.getSize().width;
        int hs = aboutBox.getSize().height;
        int xoffset = this.getX();
        int yoffset = this.getY();
        int xs = (this.getSize().width - ws) / 2 + xoffset;
        int ys = (this.getSize().height - hs) / 2 + yoffset;
        if ((xs + ws) > wscreen) {
            // past width pull back to allign with rHs
            xs = wscreen - ws;
        }
        if ((ys + hs) > hscreen) {
            // past height pull back to allign with bottom
            ys = hscreen - hs;
        }
        if (xs < 0) {
            xs = 0;
        }
        if (ys < 0) {
            ys = 0;
        }
        aboutBox.setLocation(xs, ys);
        aboutBox.setVisible(true);
    }

    private void addMousePosition(int x, int y) {
        if (mouseBytesIndex < mouseBytes.length) {
            mouseBytes[mouseBytesIndex++] = (byte) x;
        }
        if (mouseBytesIndex < mouseBytes.length) {
            mouseBytes[mouseBytesIndex++] = (byte) y;
        }
        if (mouseBytesIndex >= mouseBytes.length) {
            okText();
            errorMsgField.setText(AnotherKeyMsg);
            jProgressBar1.setValue(100);
            if (!keyGenerated) {
                keyGenerated = true;
                SwingUtilities.invokeLater(new Runnable() {
                    public void run() {
                        try {
                            generateKeyButtonActionPerformed(null);

                        } catch (Throwable t) {
                            System.err.print(t.getMessage());
                            setErrorText("Error generating key. " + t.getMessage());
                        }
                    }
                });
            }
        } else {
            jProgressBar1.setValue((mouseBytesIndex * 100) / mouseBytes.length);
            //  System.out.println("x:"+(0xff &x)+" y:"+(0xff & y));
        }
    }

    private void saveQRImage(String code) {
        ByteArrayOutputStream out = QRCode.from(""//
                //+ ApplicationSettings.SSID_LABEL//
                + ApplicationSettings.SSID + "\n"//
                //+ ApplicationSettings.SECURITY_LABEL//
                + ApplicationSettings.SECURITY + "\n"//
                //+ ApplicationSettings.PASSWORD_LABEL + "\n"//
                + code + "\n"//
                + ApplicationSettings.PORTNO_LABEL//
                + ApplicationSettings.PORTNO + "\n"//
                //+ ApplicationSettings.FEATURES_LABEL //
                //+ modeComboBox.getSelectedItem() //
                //+ " "//
                //+ ipComboBox.getSelectedItem()//
                //+ "\n"//
                //+ "website: " //
                + ApplicationSettings.thePfodAddress + "\n")//
                .to(ImageType.PNG).stream();

        try {
            FileOutputStream fout = new FileOutputStream(new File(
                    ApplicationSettings.SSID + "_" + code + ".PNG"));

            fout.write(out.toByteArray());

            fout.flush();
            fout.close();
        } catch (Throwable t) {
            System.err.print(t.getMessage());
            setErrorText("Error writing QRcode .PNG file.");
        }
        writeQRcodeButton.setEnabled(false);
    }

    /**
     * Creates new form MainFrame
     */
    public MainFrame() {
        time1 = System.nanoTime();

        initComponents();
        menuBar = new javax.swing.JMenuBar();
        menuBar.setName("menuBar"); // NOI18N

        fileMenu.setIcon(null);
        fileMenu.setText("File"); // NOI18N
        fileMenu.setName("fileMenu"); // NOI18N

        exitMenuItem.setAction(new AbstractAction() {

            public void actionPerformed(ActionEvent e) {
                exit();
            }
        });//reload); // NOI18N
        exitMenuItem.setIcon(null); // NOI18N
        exitMenuItem.setText("Exit"); // NOI18N
        exitMenuItem.setName("exitMenuItem"); // NOI18N
        fileMenu.add(exitMenuItem);

        menuBar.add(fileMenu);

        helpMenu.setIcon(null); // NOI18N
        helpMenu.setText("Help"); // NOI18N
        helpMenu.setName("helpMenu"); // NOI18N

        aboutMenuItem.setAction(new AbstractAction() {

            public void actionPerformed(ActionEvent e) {
                showAboutBox();
            }
        });
        aboutMenuItem.setText("About ..."); // NOI18N        
        aboutMenuItem.setName("aboutMenuItem"); // NOI18N
        helpMenu.add(aboutMenuItem);

        menuBar.add(helpMenu);
        setMenuBar(menuBar);

        createPastePopupMenu();
        createCopyPopupMenu();
        hashResult.addMouseListener(new MouseHandler(popupCopyMenu));
        inputTextArea.addMouseMotionListener(new MouseMovementListener());
        clearErrorText();
        writeQRcodeButton.setEnabled(false);
        generateKeyButton.setEnabled(false);
        jProgressBar1.setValue(0);

    }

    public void okText() {
        errorMsgField.setText(OKMsg);
        errorMsgField.setForeground(Color.blue);
        generateKeyButton.setEnabled(true);
    }

    public void clearErrorText() {
        errorMsgField.setText(StandardMsg);
        errorMsgField.setForeground(Color.black);
    }

    public void setErrorText(final String errorMsg) {
        generateKeyButton.setEnabled(false);
        errorMsgField.setText(errorMsg);
        errorMsgField.setForeground(Color.red);
    }

    private void createCopyPopupMenu() {
        javax.swing.JMenuItem copy = new javax.swing.JMenuItem("copy");

        popupCopyMenu = new javax.swing.JPopupMenu();
        popupCopyMenu.add(copy);

        copy.addActionListener(new cmdCopyActionListener());
    }

    class cmdCopyActionListener implements ActionListener {

        @Override
        public void actionPerformed(ActionEvent event) {
            StringSelection fieldContent = new StringSelection(hashResult.getText());
            clipboard.setContents(fieldContent, MainFrame.this);
        }
    }

    private void createPastePopupMenu() {
        javax.swing.JMenuItem paste = new javax.swing.JMenuItem("paste");

        popupPasteMenu = new javax.swing.JPopupMenu();
        popupPasteMenu.add(paste);

        paste.addActionListener(new cmdPasteActionListener());
    }

    class MouseHandler extends java.awt.event.MouseAdapter {

        private javax.swing.JPopupMenu popupMenu;

        public MouseHandler(javax.swing.JPopupMenu popup) {
            this.popupMenu = popup;
        }

        public void mousePressed(java.awt.event.MouseEvent e) {
            if (e.isPopupTrigger()) {
                popupMenu.show(e.getComponent(), e.getX(), e.getY());
            }
        }

        public void mouseReleased(java.awt.event.MouseEvent e) {
            if (e.isPopupTrigger()) {
                popupMenu.show(e.getComponent(), e.getX(), e.getY());
            }
        }
    }

    class MouseMovementListener implements MouseMotionListener {

        @Override
        public void mouseDragged(MouseEvent e) {
            // ignore
        }

        @Override
        public void mouseMoved(MouseEvent e) {
            addMousePosition(e.getX(), e.getY());
        }

    }

    class cmdPasteActionListener implements ActionListener {

        @Override
        public void actionPerformed(ActionEvent event) {

            Transferable clipboardContent = clipboard.getContents(this);

            if ((clipboardContent != null)
                    && (clipboardContent.isDataFlavorSupported(DataFlavor.stringFlavor))) {
                try {
                    String tempString;
                    tempString = (String) clipboardContent.getTransferData(DataFlavor.stringFlavor);
                    inputTextArea.setText(inputTextArea.getText() + tempString); // forces call to remove with lenght 0 first
                } catch (Throwable t) {
                }
            }
        }
    }

    private static final char[] mapping = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789()".toCharArray();

    private static int convertThree8Bytes(int three8Bytes, char[] rtn, int idx) {
        rtn[idx++] = mapping[0x3f & (three8Bytes >> 18)];
        rtn[idx++] = mapping[0x3f & (three8Bytes >> 12)];
        rtn[idx++] = mapping[0x3f & (three8Bytes >> 6)];
        rtn[idx++] = mapping[0x3f & three8Bytes];
        return idx;
    }

    public static int unsignedByteToInt(byte b) {
        return (0x000000FF & ((int) b));
    }

    public static char[] convertBytesTo64BitChars(byte[] bytesToConvert) {
        int bytesToConvertLen = bytesToConvert.length;
        char[] rtn = new char[(bytesToConvertLen / 3) * 4];
        int idx = 0;
        // take the bytes in sets of 3 and convert to 4 chars
        int bytesToConvertIdx = 0;
        bytesToConvertLen = (bytesToConvertLen/3) * 3;
        while (bytesToConvertIdx < bytesToConvertLen) {
            // use 0xff to clear any extended sign
            int three8Bytes = 0;
            three8Bytes += unsignedByteToInt(bytesToConvert[bytesToConvertIdx++]) << 16;
            three8Bytes += unsignedByteToInt(bytesToConvert[bytesToConvertIdx++]) << 8;
            three8Bytes += unsignedByteToInt(bytesToConvert[bytesToConvertIdx++]) << 0;
            // now fill in the 4 6bit bytes
            idx = convertThree8Bytes(three8Bytes, rtn, idx);
        }
        return rtn;
    }

    /**
     * This method is called from within the constructor to initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is always
     * regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jLabel1 = new javax.swing.JLabel();
        hashResult = new javax.swing.JTextField();
        jScrollPane1 = new javax.swing.JScrollPane();
        inputTextArea = new javax.swing.JTextArea();
        generateKeyButton = new javax.swing.JButton();
        errorMsgField = new javax.swing.JLabel();
        writeQRcodeButton = new javax.swing.JButton();
        jProgressBar1 = new javax.swing.JProgressBar();
        SSID_Label = new javax.swing.JLabel();
        SSID_field = new javax.swing.JLabel();
        Security_Label = new javax.swing.JLabel();
        Security_field = new javax.swing.JLabel();
        PortNo_Label = new javax.swing.JLabel();
        PortNo_field = new javax.swing.JLabel();

        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
        setFont(new java.awt.Font("Aharoni", 2, 10)); // NOI18N

        jLabel1.setText("Password:");

        hashResult.setEditable(false);
        hashResult.setVerifyInputWhenFocusTarget(false);

        inputTextArea.setEditable(false);
        inputTextArea.setColumns(20);
        inputTextArea.setRows(1);
        inputTextArea.setTabSize(2);
        inputTextArea.setText("    This application generates 'random' 128bit secret keys as 32 hex chars\n    The QRcode file is written to the same directory the pfodQRpsk.jar file is in\n    The QRcode image contains the SSID, Security, Password, PortNo and Supported Features\n    \n    Move your mouse in the text window to initialize the key generator.\n\n    The HMAC-SHA256 of your mouse movements and the current computer's nano second time will be displayed\n    The result is essential random and not reproduceable a second time\n    even using the exactly the same mouse movements.\" ");
        jScrollPane1.setViewportView(inputTextArea);

        generateKeyButton.setText("Generate Another Key");
        generateKeyButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                generateKeyButtonActionPerformed(evt);
            }
        });

        errorMsgField.setFont(new java.awt.Font("Tahoma", 3, 14)); // NOI18N
        errorMsgField.setText("Move the mouse in the text area below to initialize the key gennerator.");

        writeQRcodeButton.setText("Write QRcode .PNG");
        writeQRcodeButton.setToolTipText("");
        writeQRcodeButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                writeQRcodeButtonActionPerformed(evt);
            }
        });

        jProgressBar1.setFocusable(false);

        SSID_Label.setText("Network SSID:");
        SSID_Label.setFocusable(false);
        SSID_Label.setInheritsPopupMenu(false);
        SSID_Label.setRequestFocusEnabled(false);
        SSID_Label.setVerifyInputWhenFocusTarget(false);

        SSID_field.setFont(new java.awt.Font("Tahoma", 3, 14)); // NOI18N
        SSID_field.setText("pfodWifiConfigV1");
        SSID_field.setInheritsPopupMenu(false);
        SSID_field.setRequestFocusEnabled(false);
        SSID_field.setVerifyInputWhenFocusTarget(false);

        Security_Label.setText("Security:");
        Security_Label.setFocusable(false);
        Security_Label.setInheritsPopupMenu(false);
        Security_Label.setRequestFocusEnabled(false);
        Security_Label.setVerifyInputWhenFocusTarget(false);

        Security_field.setFont(new java.awt.Font("Tahoma", 3, 14)); // NOI18N
        Security_field.setText("WPA2-PSK");
        Security_field.setInheritsPopupMenu(false);
        Security_field.setRequestFocusEnabled(false);
        Security_field.setVerifyInputWhenFocusTarget(false);

        PortNo_Label.setText("PortNo:");
        PortNo_Label.setFocusable(false);
        PortNo_Label.setInheritsPopupMenu(false);
        PortNo_Label.setRequestFocusEnabled(false);
        PortNo_Label.setVerifyInputWhenFocusTarget(false);

        PortNo_field.setFont(new java.awt.Font("Tahoma", 3, 14)); // NOI18N
        PortNo_field.setText("23");
        PortNo_field.setInheritsPopupMenu(false);
        PortNo_field.setRequestFocusEnabled(false);
        PortNo_field.setVerifyInputWhenFocusTarget(false);

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addGap(38, 38, 38)
                .addComponent(errorMsgField, javax.swing.GroupLayout.PREFERRED_SIZE, 530, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
            .addGroup(layout.createSequentialGroup()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(layout.createSequentialGroup()
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addGroup(layout.createSequentialGroup()
                                .addGap(15, 15, 15)
                                .addComponent(SSID_Label)
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                .addComponent(SSID_field, javax.swing.GroupLayout.PREFERRED_SIZE, 137, javax.swing.GroupLayout.PREFERRED_SIZE)
                                .addGap(18, 18, 18)
                                .addComponent(Security_Label)
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                .addComponent(Security_field, javax.swing.GroupLayout.PREFERRED_SIZE, 89, javax.swing.GroupLayout.PREFERRED_SIZE)
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                .addComponent(PortNo_Label)
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                .addComponent(PortNo_field, javax.swing.GroupLayout.PREFERRED_SIZE, 27, javax.swing.GroupLayout.PREFERRED_SIZE))
                            .addGroup(layout.createSequentialGroup()
                                .addContainerGap()
                                .addComponent(jLabel1)
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                .addComponent(hashResult, javax.swing.GroupLayout.PREFERRED_SIZE, 214, javax.swing.GroupLayout.PREFERRED_SIZE)
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                                .addComponent(generateKeyButton)
                                .addGap(37, 37, 37)
                                .addComponent(writeQRcodeButton, javax.swing.GroupLayout.PREFERRED_SIZE, 157, javax.swing.GroupLayout.PREFERRED_SIZE)))
                        .addGap(0, 0, Short.MAX_VALUE))
                    .addGroup(layout.createSequentialGroup()
                        .addContainerGap()
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(jScrollPane1)
                            .addComponent(jProgressBar1, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addGap(6, 6, 6)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(SSID_Label)
                    .addComponent(SSID_field)
                    .addComponent(Security_Label)
                    .addComponent(Security_field)
                    .addComponent(PortNo_Label)
                    .addComponent(PortNo_field))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel1)
                    .addComponent(hashResult, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(generateKeyButton)
                    .addComponent(writeQRcodeButton))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(errorMsgField)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jProgressBar1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jScrollPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 237, Short.MAX_VALUE)
                .addContainerGap())
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void generateKeyButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_generateKeyButtonActionPerformed
        byte[] bytes = new byte[this.minBytes];
        System.arraycopy(mouseBytes, 0, bytes, 0, bytes.length);
        HashMap map = new HashMap();
        long time2 = System.nanoTime();
        String key = ("" + time2) + ("-" + time1);
        //System.out.println(key);
        map.put(IMac.MAC_KEY_MATERIAL, key.getBytes());
        map.put(HMac.USE_WITH_PKCS5_V2, true);

        IMac mac = HMacFactory.getInstance(Registry.HMAC_NAME_PREFIX + Registry.SHA256_HASH);
        try {
            mac.init(map);
        } catch (Throwable t) {
            System.err.println(" Error:");
            System.err.println(t.getMessage());
            setErrorText("Error processing initializing " + Registry.HMAC_NAME_PREFIX + Registry.SHA256_HASH);
            return;
        }
        mac.update(bytes, 0, bytes.length);
        byte[] hash = mac.digest();
        // take the first 128bits = 16 bytes
        byte[] resultBytes = new byte[18]; //
        int copyLen = Math.min(16, hash.length); // copy at most 16 bytes 128bits
        System.arraycopy(hash, 0, resultBytes, 0, copyLen);
        // pad with zeros
        for (int i = copyLen; i < resultBytes.length; i++) {
            resultBytes[i] = 0;
        }
        StringBuilder builder = new StringBuilder(24);

        char[] chars = convertBytesTo64BitChars(resultBytes);
        for (int i=0; (i<22 && i<chars.length);i++) {
            builder.append(chars[i]);
        }
        builder.append('p');
        hashCodeResult = builder.toString();
        hashResult.setText(hashCodeResult);
        writeQRcodeButton.setEnabled(true);
        errorMsgField.setText(AnotherKeyMsg);

    }//GEN-LAST:event_generateKeyButtonActionPerformed

    private void writeQRcodeButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_writeQRcodeButtonActionPerformed
        this.saveQRImage(hashCodeResult);
    }//GEN-LAST:event_writeQRcodeButtonActionPerformed


    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JLabel PortNo_Label;
    public javax.swing.JLabel PortNo_field;
    private javax.swing.JLabel SSID_Label;
    public javax.swing.JLabel SSID_field;
    private javax.swing.JLabel Security_Label;
    public javax.swing.JLabel Security_field;
    private javax.swing.JLabel errorMsgField;
    private javax.swing.JButton generateKeyButton;
    private javax.swing.JTextField hashResult;
    public javax.swing.JTextArea inputTextArea;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JProgressBar jProgressBar1;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JButton writeQRcodeButton;
    // End of variables declaration//GEN-END:variables

    @Override
    public void lostOwnership(Clipboard clipboard, Transferable contents) {
        // throw new UnsupportedOperationException("Not supported yet.");
    }
}
